/**
 * Tutorial example: using Ogma with JanusGraph
 *
 * This file must be build before being included in a Web page.
 * To build this page run the "npm install" command in th current directory.
 *
 * The built version of this file will be generated in "dist/index.bundle.js"
 * to be used by index.html.
 */

import Ogma from '@linkurious/ogma';

class JanusExample {
  constructor() {
    this.Gremlin = gremlin;
    /**
     * @type {Ogma}
     */
    this.ogma = new Ogma({ container: 'ogma-container' });

    // define style rules once and for all
    this.ogma.styles.addNodeRule({
      text: {
        content: this.ogma.rules.template('{{properties.name}}'),
        //content: n => n.getData(['properties', 'name']),
        color: '#333'
      },

      color: this.ogma.rules.map({
        field: ['categories'],
        values: {
          COMPANY: '#ffbb55', // companies in orange
          PERSON: '#4488ff' // persons in blue
        },
        fallback: '#666' // other nodes in gray
      }),

      radius: this.ogma.rules.map({
        field: ['categories'],
        values: {
          COMPANY: 7, // companies: bigger size
          PERSON: 5 // persons: regular size
        },
        fallback: 5
      })
    });
  }

  /**
   * @param host
   * @param port
   * @param mock
   * @return {Promise.<undefined>}
   */
  loadGraph(host, port, mock) {
    this.ogma.clearGraph();

    return this.getGraph(host, port, mock)
      .then(graph => {
        console.log('graph parsed, ok');

        return this.ogma.addGraph(graph, { locate: {} }).then(() => {
          console.log(
            `${graph.nodes.length} nodes and ${graph.edges.length} edges added, ok`
          );
        });
      })
      .then(() => {
        return this.ogma.layouts.forceLink({});
      })
      .then(() => {
        console.log('layout done');

        this.ogma.tools.legend.enable({ position: 'bottom' });

        return this.ogma.view.locateGraph({ duration: 1000 });
      })
      .catch(e => {
        alert('Error: ' + e.message);
      });
  }

  /**
   * @param host
   * @param port
   * @param mock
   * @return {Promise.<{nodes: object[], edges: object[]}>}
   */
  getGraph(host, port, mock) {
    return new Promise((resolve, reject) => {
      if (mock) {
        console.log(
          'not connecting to a Gremlin server, using a mock response'
        );
        return resolve(this.mockResponse());
      }

      console.log(`connecting to ws://${host}:${port}`);
      const client = this.Gremlin.createClient(port, host, {});

      // return all nodes and edges in the graph:
      // - get the first 10 edges, store then as "e"
      // - get the first 10 vertices, store then as "v"
      // - cap "e" and "v" (wait until the streams are completely traversed)
      const query =
        'g.V().limit(10).store("v").bothE().store("e").cap("e", "v")';
      client.execute(query, {}, (err, res) => {
        console.log('got response from gremlin server');

        if (err) {
          reject(err);
        }

        resolve(res);
      });
    }).then(gremlinResult => {
      return Ogma.parse.janus(gremlinResult);
    });
  }

  mockResponse() {
    return [
      {
        e: [
          {
            id: 'odxc8-37k-27th-3a8',
            label: 'WORKS_FOR',
            type: 'edge',
            inVLabel: 'COMPANY',
            outVLabel: 'PERSON',
            inV: 4256,
            outV: 4160,
            properties: {
              since: 2016
            }
          },
          {
            id: 'odxct-3c8-27th-3a8',
            label: 'WORKS_FOR',
            type: 'edge',
            inVLabel: 'COMPANY',
            outVLabel: 'PERSON',
            inV: 4256,
            outV: 4328,
            properties: {
              since: 2014
            }
          },
          {
            id: 'odxr1-3c8-29ed-38o',
            label: 'KNOWS',
            type: 'edge',
            inVLabel: 'PERSON',
            outVLabel: 'PERSON',
            inV: 4200,
            outV: 4328
          },
          {
            id: 'ody50-6g0-27th-3a8',
            label: 'WORKS_FOR',
            type: 'edge',
            inVLabel: 'COMPANY',
            outVLabel: 'PERSON',
            inV: 4256,
            outV: 8352,
            properties: {
              since: 2016
            }
          },
          {
            id: 'odxcg-39c-27th-380',
            label: 'WORKS_FOR',
            type: 'edge',
            inVLabel: 'COMPANY',
            outVLabel: 'PERSON',
            inV: 4176,
            outV: 4224,
            properties: {
              since: 2011
            }
          },
          {
            id: 'odxck-9ls-27th-3a8',
            label: 'WORKS_FOR',
            type: 'edge',
            inVLabel: 'COMPANY',
            outVLabel: 'PERSON',
            inV: 4256,
            outV: 12448,
            properties: {
              since: 2016
            }
          },
          {
            id: 'odxqs-9ls-29ed-38o',
            label: 'KNOWS',
            type: 'edge',
            inVLabel: 'PERSON',
            outVLabel: 'PERSON',
            inV: 4200,
            outV: 12448
          }
        ],
        v: [
          {
            id: 4160,
            label: 'PERSON',
            type: 'vertex',
            properties: {
              name: [
                {
                  id: '16w-37k-1l1',
                  value: 'Maxime'
                }
              ],
              age: [
                {
                  id: '1l4-37k-4qt',
                  value: 36
                }
              ]
            }
          },
          {
            id: 4256,
            label: 'COMPANY',
            type: 'vertex',
            properties: {
              country: [
                {
                  id: '1zo-3a8-35x',
                  value: 'France'
                }
              ],
              city: [
                {
                  id: '1lg-3a8-2dh',
                  value: 'Paris'
                }
              ],
              name: [
                {
                  id: '178-3a8-1l1',
                  value: 'Linkurious'
                }
              ]
            }
          },
          {
            id: 4176,
            label: 'COMPANY',
            type: 'vertex',
            properties: {
              country: [
                {
                  id: '2dm-380-35x',
                  value: 'USA'
                }
              ],
              city: [
                {
                  id: '2ru-380-2dh',
                  value: 'Mountain View'
                }
              ],
              name: [
                {
                  id: '362-380-1l1',
                  value: 'Google'
                }
              ]
            }
          },
          {
            id: 4200,
            label: 'PERSON',
            type: 'vertex',
            properties: {
              name: [
                {
                  id: '1l9-38o-1l1',
                  value: 'Ben'
                }
              ],
              age: [
                {
                  id: '171-38o-4qt',
                  value: '33'
                }
              ]
            }
          },
          {
            id: 4328,
            label: 'PERSON',
            type: 'vertex',
            properties: {
              name: [
                {
                  id: '17h-3c8-1l1',
                  value: 'David'
                }
              ],
              age: [
                {
                  id: '1lp-3c8-4qt',
                  value: 33
                }
              ]
            }
          },
          {
            id: 8352,
            label: 'PERSON',
            type: 'vertex',
            properties: {
              name: [
                {
                  id: '36c-6g0-1l1',
                  value: 'Francesco'
                }
              ],
              age: [
                {
                  id: '3kk-6g0-4qt',
                  value: 25
                }
              ]
            }
          },
          {
            id: 4224,
            label: 'PERSON',
            type: 'vertex',
            properties: {
              name: [
                {
                  id: '174-39c-1l1',
                  value: 'Josh'
                }
              ],
              age: [
                {
                  id: '1lc-39c-4qt',
                  value: 25
                }
              ]
            }
          },
          {
            id: 12448,
            label: 'PERSON',
            type: 'vertex',
            properties: {
              name: [
                {
                  id: '4r8-9ls-1l1',
                  value: 'Margot'
                }
              ],
              age: [
                {
                  id: '55g-9ls-4qt',
                  value: 31
                }
              ]
            }
          }
        ]
      }
    ];
  }
}

window.document.addEventListener('DOMContentLoaded', () => {
  const janusExample = new JanusExample();

  const displayGraph = () => {
    const port = window.document.getElementById('port').value;
    const host = window.document.getElementById('host').value;
    const mock = window.document.getElementById('mock').checked;
    janusExample.loadGraph(host, port, mock);
  };

  displayGraph();

  window.document.forms.gremlin.addEventListener('submit', displayGraph);
});
